//+------------------------------------------------------------------+
//|                                              MaxTrades.mq4      |
//|                      Copyright 2023, EA Coder                    |
//|                                 https://www.ea-coder.com        |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, EA Coder"
#property link      "https://www.ea-coder.com"
#property description "Limit the maximum number of trades on the account"
#property version   "1.02"
#property strict

#include <WinUser32.mqh>
#import "user32.dll"
int GetAncestor(int, int);
#define MT4_WMCMD_EXPERTS  33020
#import

// Input parameters
extern int MaxTrades = 5; // Change this value to set the maximum number of trades

// Global variable
string globalVarName = "MaxTrades_AllowTrading"; // Name of the global variable to store the trading permission

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
    // Check the global variable to see if trading is allowed from other EAs
    double varValue = GlobalVariableGet(globalVarName);
    if (varValue == 0)
    {
        int result = GlobalVariableSet(globalVarName, true);
        if (result == 0)
        {
            Print("Error while accessing the global variable. Error code: ", GetLastError());
        }
    }

    // Refresh the rates to update the trading permission
    RefreshRates();

    return INIT_SUCCEEDED;
}

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
    int totalOrders = OrdersTotal();
    bool tradingAllowed = true;
    int main = GetAncestor(WindowHandle(Symbol(), Period()), 2/*GA_ROOT*/);

    if (totalOrders >= MaxTrades)
    {
        tradingAllowed = false;
    }

    if (tradingAllowed != TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
    {
        if (tradingAllowed)
        {
            Print("Auto Trading::::: Starting AutoTrading for all terminal charts");
        }
        else
        {
            Print("Auto Trading::::: Stopping AutoTrading for all terminal charts");
        }

        PostMessageA(main, WM_COMMAND, MT4_WMCMD_EXPERTS, 0);

        // Update the global variable with the current trading permission
        int result = GlobalVariableSet(globalVarName, tradingAllowed);
        if (result == 0)
        {
            Print("Error while setting the global variable. Error code: ", GetLastError());
        }
    }
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
    // Reset the global variable on deinitialization
    int result = GlobalVariableSet(globalVarName, true);
    if (result == 0)
    {
        Print("Error while setting the global variable. Error code: ", GetLastError());
    }
}
